@@ -32,7 +32,10 @@ def get_type(size):
     8: pwndbg.typeinfo.uint64,
     }[size]
 
-@pwndbg.commands.ParsedCommand
+parser = argparse.ArgumentParser(description="Starting at the specified address, dump N bytes.")
+parser.add_argument("address", type=int, help="The address to dump from.")
+parser.add_argument("count", type=int, default=64, nargs="?", help="The number of bytes to dump.")
+@pwndbg.commands.ArgparsedCommand(parser)
 @pwndbg.commands.OnlyWhenRunning
 def db(address, count=64):
     """
@@ -41,7 +44,11 @@ def db(address, count=64):
     """
     return dX(1, (address), (count))
 
-@pwndbg.commands.ParsedCommand
+
+parser = argparse.ArgumentParser(description="Starting at the specified address, dump N words.")
+parser.add_argument("address", type=int, help="The address to dump from.")
+parser.add_argument("count", type=int, default=32, nargs="?", help="The number of words to dump.")
+@pwndbg.commands.ArgparsedCommand(parser)
 @pwndbg.commands.OnlyWhenRunning
 def dw(address, count=32):
     """
@@ -50,7 +57,11 @@ def dw(address, count=32):
     """
     return dX(2, (address), (count))
 
-@pwndbg.commands.ParsedCommand
+
+parser = argparse.ArgumentParser(description="Starting at the specified address, dump N dwrods.")
+parser.add_argument("address", type=int, help="The address to dump from.")
+parser.add_argument("count", type=int, default=16, nargs="?", help="The number of dwords to dump.")
+@pwndbg.commands.ArgparsedCommand(parser)
 @pwndbg.commands.OnlyWhenRunning
 def dd(address, count=16):
     """
@@ -59,7 +70,10 @@ def dd(address, count=16):
     """
     return dX(4, (address), (count))
 
-@pwndbg.commands.ParsedCommand
+parser = argparse.ArgumentParser(description="Starting at the specified address, dump N qwords.")
+parser.add_argument("address", type=int, help="The address to dump from.")
+parser.add_argument("count", type=int, default=8, nargs="?", help="The number of qwords to dump.")
+@pwndbg.commands.ArgparsedCommand(parser)
 @pwndbg.commands.OnlyWhenRunning
 def dq(address, count=8):
     """
@@ -68,7 +82,10 @@ def dq(address, count=8):
     """
     return dX(8, (address), (count))
 
-@pwndbg.commands.ParsedCommand
+parser = argparse.ArgumentParser(description="Starting at the specified address, hexdump.")
+parser.add_argument("address", type=int, help="The address to dump from.")
+parser.add_argument("count", type=int, default=8, nargs="?", help="The number of bytes to hexdump.")
+@pwndbg.commands.ArgparsedCommand(parser)
 @pwndbg.commands.OnlyWhenRunning
 def dc(address, count=8):
     return pwndbg.commands.hexdump.hexdump(address=address, count=count)
@@ -116,49 +133,71 @@ def enhex(size, value):
     return x
 
 
-@pwndbg.commands.Command
+parser = argparse.ArgumentParser(description="Write hex bytes at the specified address.")
+parser.add_argument("address", type=int, help="The address to write to.")
+parser.add_argument("data", type=str, nargs="*", help="The bytes to write.")
+@pwndbg.commands.ArgparsedCommand(parser)
 @pwndbg.commands.OnlyWhenRunning
-def eb(address, *data):
+def eb(address, data):
     """
     Write hex bytes at the specified address.
     """
     return eX(1, address, data)
 
-@pwndbg.commands.Command
+
+parser = argparse.ArgumentParser(description="Write hex words at the specified address.")
+parser.add_argument("address", type=int, help="The address to write to.")
+parser.add_argument("data", type=str, nargs="*", help="The words to write.")
+@pwndbg.commands.ArgparsedCommand(parser)
 @pwndbg.commands.OnlyWhenRunning
-def ew(address, *data):
+def ew(address, data):
     """
     Write hex words at the specified address.
     """
     return eX(2, address, data)
 
-@pwndbg.commands.Command
+
+parser = argparse.ArgumentParser(description="Write hex dwords at the specified address.")
+parser.add_argument("address", type=int, help="The address to write to.")
+parser.add_argument("data", type=str, nargs="*", help="The dwords to write.")
+@pwndbg.commands.ArgparsedCommand(parser)
 @pwndbg.commands.OnlyWhenRunning
-def ed(address, *data):
+def ed(address, data):
     """
     Write hex dwords at the specified address.
     """
     return eX(4, address, data)
 
-@pwndbg.commands.Command
+
+parser = argparse.ArgumentParser(description="Write hex qwords at the specified address.")
+parser.add_argument("address", type=int, help="The address to write to.")
+parser.add_argument("data", type=str, nargs="*", help="The qwords to write.")
+@pwndbg.commands.ArgparsedCommand(parser)
 @pwndbg.commands.OnlyWhenRunning
-def eq(address, *data):
+def eq(address, data):
     """
     Write hex qwords at the specified address.
     """
     return eX(8, address, data)
 
-@pwndbg.commands.Command
-@pwndbg.commands.OnlyWhenRunning
-def ez(address, *data):
-    """
-    Write a string at the specified address.
-    """
-    return eX(1, address, data[0], hex=False)
 
-@pwndbg.commands.Command
+parser = argparse.ArgumentParser(description="Write a string at the specified address.")
+parser.add_argument("address", type=int, help="The address to write to.")
+parser.add_argument("data", type=str, help="The string to write.")
+@pwndbg.commands.ArgparsedCommand(parser)
 @pwndbg.commands.OnlyWhenRunning
-def eza(address, *data):
+def ez(address, data):
+    """
+    Write a character at the specified address.
+    """
+    return eX(1, address, data, hex=False)
+
+parser = argparse.ArgumentParser(description="Write a string at the specified address.") #TODO Is eza just ez? If so just alias. I had trouble finding windbg documentation defining ez
+parser.add_argument("address", type=int, help="The address to write to.")
+parser.add_argument("data", type=str, help="The string to write.")
+@pwndbg.commands.ArgparsedCommand(parser)
+@pwndbg.commands.OnlyWhenRunning
+def eza(address, data):
     """
     Write a string at the specified address.
     """
@@ -191,37 +230,15 @@ def eX(size, address, data, hex=True):
 
         pwndbg.memory.write(address + (i * size), data)
 
-@pwndbg.commands.ParsedCommand
+parser = argparse.ArgumentParser(description="Dump pointers and symbols at the specified address.")
+parser.add_argument("addr", type=int, help="The address to dump from.")
+@pwndbg.commands.ArgparsedCommand(parser,aliases=['kd','dps','dqs']) #TODO are these really all the same? They had identical implementation...
 @pwndbg.commands.OnlyWhenRunning
-def dds(*a):
+def dds(addr):
     """
     Dump pointers and symbols at the specified address.
     """
-    return pwndbg.commands.telescope.telescope(*a)
-
-@pwndbg.commands.ParsedCommand
-@pwndbg.commands.OnlyWhenRunning
-def kd(*a):
-    """
-    Dump pointers and symbols at the specified address.
-    """
-    return pwndbg.commands.telescope.telescope(*a)
-
-@pwndbg.commands.ParsedCommand
-@pwndbg.commands.OnlyWhenRunning
-def dps(*a):
-    """
-    Dump pointers and symbols at the specified address.
-    """
-    return pwndbg.commands.telescope.telescope(*a)
-
-@pwndbg.commands.ParsedCommand
-@pwndbg.commands.OnlyWhenRunning
-def dqs(*a):
-    """
-    Dump pointers and symbols at the specified address.
-    """
-    return pwndbg.commands.telescope.telescope(*a)
+    return pwndbg.commands.telescope.telescope(addr)
 
 
 da_parser = argparse.ArgumentParser()
@@ -248,14 +265,18 @@ def ds(address, max):
     address &= pwndbg.arch.ptrmask
     print("%x" % address, repr(pwndbg.strings.get(address, max)))
 
-@pwndbg.commands.ParsedCommand
+
+@pwndbg.commands.ArgparsedCommand("List breakpoints.")
 def bl():
     """
     List breakpoints
     """
     gdb.execute('info breakpoints')
 
-@pwndbg.commands.Command
+
+parser = argparse.ArgumentParser(description="Disable the breakpoint with the specified index.")
+parser.add_argument("which", nargs="?", type=str, default='*', help="Index of the breakpoint to disable.")
+@pwndbg.commands.ArgparsedCommand(parser)
 def bd(which = '*'):
     """
     Disable the breakpoint with the specified index.
@@ -266,7 +287,9 @@ def bd(which = '*'):
         gdb.execute('disable breakpoints %s' % which)
 
 
-@pwndbg.commands.Command
+parser = argparse.ArgumentParser(description="Enable the breakpoint with the specified index.")
+parser.add_argument("which", nargs="?", type=str, default='*', help="Index of the breakpoint to enable.")
+@pwndbg.commands.ArgparsedCommand(parser)
 def be(which = '*'):
     """
     Enable the breakpoint with the specified index.
@@ -276,7 +299,9 @@ def be(which = '*'):
     else:
         gdb.execute('enable breakpoints %s' % which)
 
-@pwndbg.commands.Command
+parser = argparse.ArgumentParser(description="Clear the breakpoint with the specified index.")
+parser.add_argument("which", nargs="?", type=str, default='*', help="Index of the breakpoint to clear.")
+@pwndbg.commands.ArgparsedCommand(parser)
 def bc(which = '*'):
     """
     Clear the breakpoint with the specified index.
@@ -287,7 +312,9 @@ def bc(which = '*'):
         gdb.execute('delete breakpoints %s' % which)
 
 
-@pwndbg.commands.ParsedCommand
+parser = argparse.ArgumentParser(description="Set a breakpoint at the specified address.")
+parser.add_argument("where", type=int, help="The address to break at.")
+@pwndbg.commands.ArgparsedCommand(parser)
 def bp(where):
     """
     Set a breakpoint at the specified address.
@@ -296,7 +323,11 @@ def bp(where):
     if result is not None:
         gdb.execute('break *%#x' % int(result))
 
-@pwndbg.commands.ParsedCommand
+
+parser = argparse.ArgumentParser(description="Starting at the specified address, disassemble N instructions.")
+parser.add_argument("where", type=int, nargs="?", default=None, help="The address to disassemble at.")
+parser.add_argument("n", type=int, nargs="?", default=5, help="The number of instructions to disassemble.")
+@pwndbg.commands.ArgparsedCommand(parser)
 @pwndbg.commands.OnlyWhenRunning
 def u(where=None, n=5, to_string=False):
     """
@@ -307,7 +338,7 @@ def u(where=None, n=5, to_string=False):
         where = pwndbg.regs.pc
     return pwndbg.commands.nearpc.nearpc(where, n, to_string)
 
-@pwndbg.commands.Command
+@pwndbg.commands.ArgparsedCommand("Print a backtrace (alias 'bt').")
 @pwndbg.commands.OnlyWhenRunning
 def k():
     """
@@ -315,16 +346,21 @@ def k():
     """
     gdb.execute('bt')
 
-@pwndbg.commands.ParsedCommand
+
+parser = argparse.ArgumentParser(description="List the symbols nearest to the provided value.")
+parser.add_argument("value", type=int, nargs="?", default=None, help="The address you want the name of.")
+@pwndbg.commands.ArgparsedCommand(parser)
 @pwndbg.commands.OnlyWhenRunning
 def ln(value=None):
     """
     List the symbols nearest to the provided value.
     """
     if value is None: value = pwndbg.regs.pc
+    value = int(value)
     x = pwndbg.symbol.get(value)
     if x:
         result = '(%#x)   %s' % (value, x)
+        print(result)
 
 @pwndbg.commands.QuietSloppyParsedCommand
 @pwndbg.commands.OnlyWhenRunning
@@ -351,12 +387,12 @@ def vprot(map):
     """
     return pwndbg.commands.vmmap.vmmap(map)
 
-@pwndbg.commands.Command
+@pwndbg.commands.ArgparsedCommand("Not be windows.")
 @pwndbg.commands.OnlyWhenRunning
-def peb(*a):
+def peb():
     print("This isn't Windows!")
 
-@pwndbg.commands.Command
+@pwndbg.commands.ArgparsedCommand("Windbg compatibility alias for 'continue' command.")
 @pwndbg.commands.OnlyWhenRunning
 def go():
     '''
@@ -364,7 +400,7 @@ def go():
     '''
     gdb.execute('continue')
 
-@pwndbg.commands.Command
+@pwndbg.commands.ArgparsedCommand("Windbg compatibility alias for 'nextcall' command.")
 @pwndbg.commands.OnlyWhenRunning
 def pc():
     '''
