@@ -53,3 +53,11 @@ class BaseHeap(object):
             A boolean.
         """
         raise NotImplementedError()
+
+    def libc_has_debug_syms(self):
+        """Returns whether the libc has debug symbols or not.
+
+        Returns:
+            A boolean.
+        """
+        raise NotImplementedError()
