@@ -58,7 +58,7 @@ _ida_last_connection_check = 0
 
 
 @pwndbg.decorators.only_after_first_prompt()
-@pwndbg.config.Trigger([ida_rpc_host, ida_rpc_port, ida_timeout])
+@pwndbg.config.Trigger([ida_rpc_host, ida_rpc_port, ida_enabled, ida_timeout])
 def init_ida_rpc_client():
     global _ida, _ida_last_exception, _ida_last_connection_check
 
@@ -227,7 +227,8 @@ def here():
 @withIDA
 @takes_address
 def Jump(addr):
-    return _ida.Jump(addr)
+    # uses C++ api instead of idc one to avoid activating the IDA window
+    return _ida.jumpto(addr, -1, 0)
 
 
 @withIDA
@@ -391,6 +392,13 @@ def decompile(addr):
     return _ida.decompile(addr)
 
 
+@withHexrays
+@takes_address
+@pwndbg.memoize.reset_on_stop
+def decompile_context(pc, context_lines):
+    return _ida.decompile_context(pc, context_lines)
+
+
 @withIDA
 @pwndbg.memoize.forever
 def get_ida_versions():
