@@ -62,13 +62,7 @@ class module(ModuleType):
 
     @property
     def exe(self):
-        for obj in gdb.objfiles():
-            if obj.filename:
-                return obj.filename
-            break
-        if self.alive:
-            auxv = pwndbg.auxv.get()
-            return auxv['AT_EXECFN']
+        return gdb.current_progspace().filename
 
     @property
     def mem_page(self):
